var Install = function()
{
	var install = {};
	var debug = 0;
	
	install.initPage = function()
	{
		if (!App.getActiveProjectId()) return;
		Buttons.addClickHandler('btnFixItems', install.fixInstallation);
		Buttons.addClickHandler('btnCreateMissingItems', install.fixInstallation);
		
		install.checkInstallation('Validate', 'Checking installation...');
	}
	
	install.fixInstallation = function()
	{
		install.checkInstallation('Install', 'Fixing installation...');
	}

	
	install.checkInstallation = function(cmd, workText)
	{
		Doc.hide('validationResult0');
		Doc.hide('validationResult1');
		Doc.hide('validationResult2');
		Doc.hide('createMissingItems');
		Doc.hide('fixIncompleteItems');
		
		var url = '../Install.servlet?ID=' + App.getSessionId();
		url += '&cmd='+cmd;
		Doc.addClass('workInProgress', 'working');
		Wizard.showLoadingAnimation(workText);
		Wizard.asyncJsonRequest(url, install.onChecked);
	}
	
	install.onChecked = function(response)
	{
		Doc.removeClass('workInProgress', 'working');
		
		var COL_BREAKS = ['ANNOTATIONTYPE', 'PLUGINDEFINITION'];
		var checks = response.checks;
		
		var numMissing = 0;
		var numWarnings = 0;
		var numErrors = 0;
		var numIncomplete = 0;
		var html = '<table class="report">';
		var lastItemType = null;
		var index = 0;
		var topLines = []; // Top lines are for items with error/warning/special message
		var bottomLines = []; // Bottom lines are for all 'OK' items
		var allOk = true;
		var clickableItems = [];
		var showMore = [];
		
		for (var i = 0; i < checks.length; i++)
		{
			var check = checks[i];
			var icon = 'ok.png';
			check.ok = true;
			if (check.status == 'missing') 
			{
				numMissing++;
				icon = 'error.png';
				check.ok = false;
			}
			if (check.status == 'incomplete')
			{
				numIncomplete++;
				icon = 'warning.png';
				check.ok = false;
			}
			if (check.status == 'error') 
			{
				numErrors++;
				icon = 'error.png';
				check.ok = false;
			}
			if (check.status == 'warning') 
			{
				numWarnings++;
				icon = 'warning.png';
				check.ok = false;
			}
			allOk &= check.ok;
			
			var sameSection = lastItemType == check.itemType;
			// Do not start new section when switching between plugin definition and configuration
			if (!sameSection) sameSection = lastItemType == 'PLUGINDEFINITION' && check.itemType == 'PLUGINCONFIGURATION';
			if (!sameSection) sameSection = lastItemType == 'PLUGINCONFIGURATION' && check.itemType == 'PLUGINDEFINITION';
			if (!sameSection)
			{
				html += topLines.join(''); // Always show the top-lines
				if ((topLines.length + bottomLines.length > 13) && bottomLines.length > 0)
				{
					// Show 10 lines and hide the rest
					var showMoreFrom = Math.max(0, 10 - topLines.length);
					if (showMoreFrom > 0)
					{
						html += bottomLines.slice(0, showMoreFrom).join('');
					}
					showMore[showMore.length] = check.itemType;
					html += '<tr class="sameitemtype highlight" id="'+check.itemType+'">';
					html += '<td class="link">&hellip; '+(bottomLines.length-showMoreFrom)+' more</td>';
					html += '<td><img src="../images/ok.png"></td><td>Ok</td></tr></tbody>';
					html += '<tbody id="'+check.itemType+'.more" style="display: none;">';
					html += bottomLines.slice(showMoreFrom).join('');
					html += '</tbody>';
				}
				else
				{
					html += bottomLines.join('')+'</tbody>';
				}
				
				topLines = [];
				bottomLines = [];
				allOk = true;
				index = 0;
				
				var col_break = COL_BREAKS.indexOf(check.itemType);
				if (col_break >= 0)
				{
					// End the left side and start on the right
					html += '</table>';
					Doc.element('validationResult'+col_break).innerHTML = html;
					Doc.show('validationResult'+col_break);
					html = '<table class="report">';
				}
				html += '<tbody><tr class="newitemtype bg-filled-100 "><td colspan="3">'+check.itemType+'</td></tr>';
			}
			
			var line = '<tr class="sameitemtype highlight '+check.itemType+' ' + (check.ok ? 'check-ok' : 'check-not-ok') + '">';
			lastItemType = check.itemType;
			var name = check.name;
			if (check.mainType) name += ' <span class="itemsubtype">[' + check.mainType + ']</span>'; 
			if (check.id)
			{
				clickableItems[clickableItems.length] = check.itemType+check.id;
				line += '<td class="itemTypeCol"><div class="link" id="'+check.itemType+check.id+'" data-item-type="'+check.itemType+'" data-item-id="'+check.id+'"';
				line += ' title="View this item (use CTRL, ALT or SHIFT to edit)">'+name+'</div></td>';
			}
			else
			{
				line += '<td class="itemTypeCol"><i>' + name + '</i></td>';
			}
			line += '<td class="iconCol"><img src="../images/'+icon+'"></td>';
			line += '<td class="statusCol">';
			if (check.messages.length > 1)
			{
				for (var m = 0; m < check.messages.length; m++)
				{
					line += '• '+check.messages[m] + '<br>';
				}
			}
			else
			{
				line += check.messages;
			}
			line += '</td></tr>';
			
			if (!check.ok || check.messages != 'Ok')
			{
				topLines[topLines.length] = line;
			}
			else
			{
				bottomLines[bottomLines.length] = line;
			}
			index++;
		}
		
		html += topLines.join('');
		if ((topLines.length + bottomLines.length > 13) && bottomLines.length > 0)
		{
			// Show 10 lines and hide the rest
			var showMoreFrom = Math.max(0, 10 - topLines.length);
			if (showMoreFrom > 0)
			{
				html += bottomLines.slice(0, showMoreFrom).join('');
			}
			showMore[showMore.length] = check.itemType;
			html += '<tr class="sameitemtype highlight" id="'+check.itemType+'">';
			html += '<td class="link">&hellip; '+(bottomLines.length-showMoreFrom)+' more</td>';
			html += '<td><img src="../images/ok.png"></td><td>Ok</td></tr></tbody>';
			html += '<tbody id="'+check.itemType+'.more" style="display: none;">';
			html += bottomLines.slice(showMoreFrom).join('');
			html += '</tbody>';
		}
		else
		{
			html += bottomLines.join('')+'</tbody>';
		}
		
		html += '</table>';
		Doc.element('validationResult'+COL_BREAKS.length).innerHTML = html;
		Doc.show('validationResult'+COL_BREAKS.length);

		// Add click handler to items
		for (var i = 0; i < clickableItems.length; i++)
		{
			Events.addEventHandler(clickableItems[i], 'click', Items.itemOnClick);
		}
		// Add click handler to 'show more...'
		for (var i = 0; i < showMore.length; i++)
		{
			Events.addEventHandler(showMore[i], 'click', install.showMore);
		}
		
		if (numErrors > 0)
		{
			Wizard.setFatalError(numErrors+' errors was detected. You need to fix those manually.');	
		}
		else if (numMissing > 0)
		{
			Doc.show('createMissingItems');
		}
		else if (numIncomplete > 0)
		{
			Doc.show('fixIncompleteItems');
		}
		else if (numWarnings > 0)
		{
			Wizard.showStatusMessage(numWarnings+' warnings was detected. Reggie may still work. If not, you need to fix it manually.');
		}
	}
	
	install.showMore = function(event)
	{
		var target = event.currentTarget;
		Doc.show(target.id + '.more');
		Doc.hide(target.id);
	}

	
	return install;
}();

Doc.onLoad(Install.initPage);
